-- AshranBuddy.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

AshranBuddy = LibStub("AceAddon-3.0"):NewAddon("AshranBuddy", "AceConsole-3.0", "AceComm-3.0", "AceSerializer-3.0")
AshranBuddy.version = 1
AshranBuddy.modules = { }

-- Constants
local ASHRAN_AREA_ID = 978;

-- Code that you want to run when the addon is first loaded goes here.
function AshranBuddy:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AshranBuddyDB", self.defaults)
	
	-- Initialize modules
	for key, module in pairs(self.modules) do
		module:Initialize()
	end
end

-- Called when the addon is enabled
function AshranBuddy:OnEnable()
	-- Register events
	self:RegisterChatCommand("ashranbuddy", "OnChatCommand")
	self:RegisterChatCommand("ab", "OnChatCommand")
	
	self:RegisterComm("AshranBuddy", self.OnMessageReceived)
		
	-- Enable modules
	for key, module in pairs(self.modules) do
		if (self.db.profile[key].enabled) then
			module:Enable()
		end
	end

	-- Create interface options
	self:CreateOptions()
	
	-- Show first time message
	local options = self.db.profile
	if (options.firstRun) then
		options.firstRun = false
		print(L["Welcome to Ashran Buddy!"])
	end
end

-- Called when the addon is disabled
function AshranBuddy:OnDisable()
    -- Unregister events
	self:UnregisterAllEvents()

	-- Disable modules
	for key, module in pairs(self.modules) do
		module:Disable()
	end
end

-- Process the slash command ('input' contains whatever follows the slash command)
function AshranBuddy:OnChatCommand(input)
	if (input == "") then
		AceDialog = AceDialog or LibStub("AceConfigDialog-3.0")
		AceRegistry = AceRegistry or LibStub("AceConfigRegistry-3.0")

		if not AshranBuddy.options then
			AshranBuddy:CreateOptions()
			AceDialog:SetDefaultSize("AshranBuddy", 800, 500)
		end

		AceDialog:Open("AshranBuddy")
	end

	-- Pass chat commands to active modules
	for key, module in pairs(self.modules) do
		if (module:IsEnabled() and type(module.OnChatCommand) == "function") then
			module:OnChatCommand(input)
		end
	end
end

-- Process messages received from other clients
function AshranBuddy:OnMessageReceived(text, distribution, sender)
    local success, packet = AshranBuddy:Deserialize(text)
	if (not success) then
		return
	end
	
	-- Backwards compatibility
	-- TODO: Remove
	if (packet.module == nil) then
		packet.module = "EventTimer"
	end

	-- Pass messages to correct module
	local module = AshranBuddy.modules[packet.module]
	if (module ~= nil and type(module.OnMessageReceived) == "function") then
		module:OnMessageReceived(packet, distribution, sender)
	end
end

function AshranBuddy:RegisterModule(name)
	local module = { }
	module.containerFrame = CreateFrame("Frame")
	module.enabled = false
	module.name = name

	-- Register the module instance
	self.modules[name] = module
	
	-- Activity registration
	module.Initialize = function(self)
		if type(self.OnInitialize) == "function" then
			self:OnInitialize()
		end
	end
	module.Enable = function(self)
		if not self.enabled then
			self.enabled = true
			if type(self.OnEnable) == "function" then
				self:OnEnable()
			end
		end
	end
	module.Disable = function(self)
		if (self.enabled) then
			self.enabled = false
			if type(self.OnDisable) == "function" then
				self:OnDisable()
			end
		end
	end
	module.IsEnabled = function(self)
		return self.enabled
	end

	-- Event registration
	module.containerFrame.events = { }
	module.containerFrame:SetScript("OnEvent", function(self, event, ...)
		local func = module.containerFrame.events[event]
		if (type(module[func]) == "function") then
			module[func](module, event, ...)
		end
	end)
	module.RegisterEvent = function(self, event, func)
		self.containerFrame.events[event] = func or event
		self.containerFrame:RegisterEvent(event)
	end
	module.UnregisterEvent = function(self, event)
		self.containerFrame.events[event] = nil
		self.containerFrame:UnregisterEvent(event)
	end
	module.UnregisterAllEvents = function(self)
		self.containerFrame:UnregisterAllEvents()
	end
	
	-- Messaging
	module.SendMessage = function(self, packet, distribution, sender)
		if (packet.module == nil) then
			packet.module = self.name
		end
		AshranBuddy:SendMessage(packet, distribution, sender)
	end

	return module
end

function AshranBuddy:SendMessage(packet, distribution, sender)
	self:SendCommMessage("AshranBuddy", self:Serialize(packet), distribution, sender, "NORMAL")
end

function AshranBuddy:IsInAshran()
	return GetCurrentMapAreaID() == ASHRAN_AREA_ID
end

function round(num, precision)
	return tonumber(string.format("%." .. (precision or 0) .. "f", num))
end