-- BonusObjectives.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["Objectives"]))
end

-- Registration
local BonusObjectives = AshranBuddy:RegisterModule("BonusObjectives")

local MINIMAP_QUEST_BONUS_OBJECTIVE = 49;
local BONUS_DAILY_QUESTS = {
	37478, --Bonus Objective: Amphitheater of Annihilation
	37476, --Bonus Objective: Ashmaul Burial Grounds
	37477, --Bonus Objective: Brute's Rise
	37475  --Bonus Objective: Molten Quarry
}

function BonusObjectives:OnInitialize()
	self:CreateContainerFrame()
end

function BonusObjectives:OnEnable()
	self:RegisterEvent("WORLD_MAP_UPDATE", "OnWorldMapUpdate")
	self:RegisterEvent("QUEST_TURNED_IN", "OnQuestTurnedIn")
end

function BonusObjectives:OnDisable()
	self:UnregisterAllEvents()
	self.containerFrame:Hide()
end

function BonusObjectives:OnWorldMapUpdate()
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
		local bonusObjectives = AshranBuddy.db.profile.BonusObjectives
		local datetime = time(date("*t"))
		for i = 1, #BonusObjectives.landmarks do
			local landmark = BonusObjectives.landmarks[i]
			local isInArea, isOnMap, numObjectives = GetTaskInfo(landmark.questID)
			if (isInArea ~= nil or datetime > bonusObjectives[landmark.questID]) then
				landmark:Show()
			else
				landmark:Hide()
			end
		end
	else
		self.containerFrame:Hide()
	end
end

function BonusObjectives:OnQuestTurnedIn(event, questID, arg1, reward)
	for i = 1, #BONUS_DAILY_QUESTS do
		if (BONUS_DAILY_QUESTS[i] == questID) then
			local datetime = time(date("*t"))
			local nextDaily = GetQuestResetTime()
			AshranBuddy.db.profile.BonusObjectives[questID] = datetime + nextDaily
		end
	end
end

function BonusObjectives:OnChatCommand(input)
end

function BonusObjectives:CreateContainerFrame()
	self.landmarks = { }

	local frame = self.containerFrame
	frame:SetParent(WorldMapButton)
	frame:SetSize(50, 50)
	frame:SetPoint("TOPLEFT", WorldMapButton, "TOPLEFT", 0, 0)
	frame:SetToplevel(true)
	frame:EnableMouse(true)
	
	self.landmarks[1] = self:CreateLandmarkFrame(frame, BONUS_DAILY_QUESTS[1], 340, 320)
	self.landmarks[2] = self:CreateLandmarkFrame(frame, BONUS_DAILY_QUESTS[2], 365, 620)
	self.landmarks[3] = self:CreateLandmarkFrame(frame, BONUS_DAILY_QUESTS[3], 620, 280)
	self.landmarks[4] = self:CreateLandmarkFrame(frame, BONUS_DAILY_QUESTS[4], 560, 700)
end

function BonusObjectives:CreateLandmarkFrame(frame, questID, x, y)
	local button = CreateFrame("Button", nil, frame)
    button:SetScript("OnEnter", self.OnLandmarkEnter)
    button:SetScript("OnLeave", self.OnLandmarkLeave)
     
    button.Texture = button:CreateTexture(nil, "BACKGROUND")
    button:SetWidth(32)
	button:SetHeight(32)
	button.Texture:SetWidth(28)
	button.Texture:SetHeight(28)
	button.Texture:SetPoint("TOPLEFT", 2, 2)
	button.Texture:SetTexture("Interface\\Minimap\\ObjectIcons")
	
	local x1, x2, y1, y2 = GetWorldEffectTextureCoords(MINIMAP_QUEST_BONUS_OBJECTIVE)
	button.Texture:SetTexCoord(x1, x2, y1, y2)
	
	button:SetPoint("TOPLEFT", WorldMapButton, "TOPLEFT", (x - 8) * WorldMapButton:GetWidth() / 1000, (y - 8) * WorldMapButton:GetHeight() / -1000)
	button.questID = questID
	return button
end

function BonusObjectives:OnLandmarkEnter()
    if(self ~= nil and self.questID ~= nil) then
        WorldMapTooltip:SetOwner(self, "ANCHOR_RIGHT")
        local name = C_TaskQuest.GetQuestTitleByQuestID(self.questID)
        local objectives = C_TaskQuest.GetQuestObjectiveStrByQuestID(self.questID)
 
        WorldMapTooltip:SetText(name)
        if (objectives ~= nil) then
            for key, value in pairs(objectives) do
                WorldMapTooltip:AddLine(QUEST_DASH..value, 1, 1, 1, true)
            end
        end
        WorldMapTooltip:Show()
    end
end
 
function BonusObjectives:OnLandmarkLeave(self)
    WorldMapFrame.poiHighlight = nil
    WorldMapFrameAreaLabel:SetText(WorldMapFrame.areaName)
    WorldMapFrameAreaDescription:SetText("")
    WorldMapTooltip:Hide()
end

function BonusObjectives:OnCreateOptions(options)
	options.BonusObjectives = {
		name = L["Objectives"],
		type = "group",
		args = {
			header = {
				name = L["Objectives"],
				type = "header",
				order = 0
			},
			headerDescription = {
				name = L["Gives you the option to track specific objectives within Ashran. More to be added soon!"],
				type = "description",
				order = 1
			},
			enabled = {
				name = L["Enabled"],
				desc = L["Enables the Objectives module."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.BonusObjectives.enabled = val; if (val) then self:Enable() else self:Disable() end end,
				get = function(info) return AshranBuddy.db.profile.BonusObjectives.enabled end,
				order = 2
			},
			enabledDescription = {
				name = L["Enables the Objectives module."],
				type = "description",
				order = 3
			},
		},
	}
end
