-- EventTimer.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["Event Timer"]))
end

-- Registration
local EventTimer = AshranBuddy:RegisterModule("EventTimer")
LibStub("AceTimer-3.0"):Embed(EventTimer)

-- Locals
local Event = { }
Event[178019] = { ID = 178019, Name = L["Collect Empowered Ore"], Title = L["Empowered Ore"], MapX = round(0.545, 2), MapY = round(0.690, 2) }
Event[178021] = { ID = 178021, Name = L["Ogre Fires"], Title = L["Ogre Fires"], MapX = 0, MapY = 0 }
Event[178020] = { ID = 178020, Name = L["Risen Spirits"], Title = L["Risen Spirits"], MapX = round(0.372, 2), MapY = round(0.631, 2) }
Event[178022] = { ID = 178022, Name = L["Stadium Racing"], Title = L["Stadium Racing"], MapX = round(0.376, 2), MapY = round(0.375, 2) }

local EventState = { }
EventState.Hidden = { ID = -1, Category = 6, Value = 0, MinValue = 0, MaxValue = 0, Red = 0, Green = 0, Blue = 0, Text = "", Message = nil }
EventState.Inactive = { ID = 0, Category = 5, Value = 0, MinValue = 0, MaxValue = 1800, Red = 0.5, Green = 0.5, Blue = 0.5, Text = nil, Message = L["%s is on cooldown for %s."] }
EventState.Intermission = { ID = 1, Category = 4, Value = 0, MinValue = 0, MaxValue = 600, Red = 1, Green = 0, Blue = 0, Text = nil, Message = L["%s will announce randomly within the next %s."] }
EventState.Announced = { ID = 2, Category = 3, Value = 0, MinValue = 0, MaxValue = 180, Red = 0.26, Green = 0.42, Blue = 1, Text = nil, Message = L["%s will begin in %s."] }
EventState.PreActive = { ID = 3, Category = 2, Value = 60, MinValue = 0, MaxValue = 60, Red = 0, Green = 1, Blue = 0, Text = "", Message = L["%s is currently active."] }
EventState.Active = { ID = 4, Category = 1, Value = 60, MinValue = 0, MaxValue = 60, Red = 0, Green = 1, Blue = 0, Text = "", Message = L["%s is currently active."] }

local EventTransition = { }
EventTransition[EventState.Inactive] = EventState.Intermission
EventTransition[EventState.Intermission] = EventState.Announced
EventTransition[EventState.Announced] = EventState.PreActive

local LandmarkTextureIndex = 42
local IsDirty = false

function EventTimer:OnInitialize()
	self:InitializeComponent(220, 65)
	self:Migrate()
end

function EventTimer:OnEnable()
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "OnChatMessageMonsterEmote")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChangedNewArea")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("GROUP_JOINED", "OnGroupJoined")
	
	if (not AshranBuddy.db.profile.EventTimer.locked) then
		EventTimer:Unlock()
	end
	
	self:SendRequest()
end

function EventTimer:OnDisable()
	self:UnregisterAllEvents()
	self.containerFrame:Hide()
	
	for i = 1, #EventTimer.statusBars do
		local statusBar = self.statusBars[i]
		statusBar:SetState(EventState.Hidden)
	end
end

function EventTimer:OnChatMessageMonsterEmote(event, message, sender, language, channelString, target, flags, unused1, channelNumber, channelName, unused2, counter)
	if (AshranBuddy:IsInAshran()) then
		for i = 1, #self.statusBars do
			local statusBar = self.statusBars[i]
			if (message:find(statusBar:GetEventID())) then
				statusBar:SetState(EventState.Announced)
			end
		end
	end
end

function EventTimer:OnZoneChangedNewArea(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
		self:SendRequest()
	else
		if (AshranBuddy.db.profile.EventTimer.locked == false) then
			self:Unlock()
		else
			self.containerFrame:Hide()
		end
		for i = 1, #EventTimer.statusBars do
			local statusBar = self.statusBars[i]
			statusBar:SetState(EventState.Hidden)
		end
	end
end

function EventTimer:OnPlayerEnteringWorld(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
		self:SendRequest()
	else
		if (AshranBuddy.db.profile.EventTimer.locked == false) then
			self:Unlock()
		else
			self.containerFrame:Hide()
		end
	end
end

function EventTimer:OnGroupJoined(event)
	if (AshranBuddy:IsInAshran()) then
		self:SendRequest()
	end
end

function EventTimer:OnUpdate()
	if (not AshranBuddy:IsInAshran()) then
		return
	end
	for i = 1, #EventTimer.statusBars do
		local statusBar = EventTimer.statusBars[i]
		local event = Event[statusBar:GetEventID()]
		local exists = EventTimer:FindWorldMapLandmark(event.Name, event.MapX, event.MapY)
		if (exists) then
			statusBar:SetState(EventState.Active)
		elseif (not exists and statusBar:GetState() == EventState.Active) then
			statusBar:SetState(EventState.Inactive)
		end
	end
	if (IsDirty and AshranBuddy.db.profile.EventTimer.stacked) then
		local frame = EventTimer.containerFrame
		local bars = { }
		for i = 1, #EventTimer.statusBars do
			bars[#bars + 1] = { EventTimer.statusBars[i]:GetOrder(), EventTimer.statusBars[i] }
		end
		table.sort(bars, function(a, b) return a[1] < b[1] end)
		
		local y = -5
		for key, value in pairs(bars) do
			value[2]:ClearAllPoints()
			value[2]:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, y)
			y = y - 15
		end
		IsDirty = false
	end
end

function EventTimer:OnMouseDown()
	if (AshranBuddy.db.profile.EventTimer.locked == false) then
		self:StartMoving()
	end
end

function EventTimer:OnMouseUp()
	if (AshranBuddy.db.profile.EventTimer.locked == false) then
		self:StopMovingOrSizing()
	end
end

function EventTimer:OnStatusBarClick(button)
	local state = self:GetState()
	local minValue, maxValue = self:GetMinMaxValues()
	local timeLeft = maxValue - self:GetValue()
	local timeText = string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60)
	local message = string.format(L["AshranBuddy: [Event: %s] "], self:GetEventTitle())
	message = string.format(state.Message, message, timeText)

	if (button == "LeftButton" and IsShiftKeyDown() and IsInRaid()) then
		SendChatMessage(message, "RAID")
	end
	if (button == "LeftButton" and IsAltKeyDown() and IsInRaid()) then
		SendChatMessage(message , "RAID_WARNING")
	end
	if (button == "LeftButton" and IsControlKeyDown()) then
		local index = GetChannelName(L["General - Ashran"])
		if (index ~= nil) then 
			SendChatMessage(message , "CHANNEL", nil, index)
		end
	end
end

function EventTimer:OnTimerCallback(statusBar, transition)
	local value = statusBar:GetValue()
	local minValue, maxValue = statusBar:GetMinMaxValues()
	local timeLeft = maxValue - value
	
	statusBar:SetEventTime(string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60))
	statusBar:SetValue(value + 1)

	if (value >= maxValue) then
		if (statusBar:GetEventTitle() == L["Ogre Fires"] and transition == EventState.PreActive) then
			-- TODO: Ogre Fires cannot be tracked correctly due to a Blizzard issue
			statusBar:SetState(EventState.Inactive)
		else
			statusBar:SetState(transition)
		end
	end
end

function EventTimer:Lock()
	if (not AshranBuddy:IsInAshran()) then
		self.containerFrame:Hide()
	end
	self.handle:Hide()
	
	local point, relativeTo, relativePoint, x, y = self.containerFrame:GetPoint(1)
	AshranBuddy.db.profile.EventTimer.point = point
	AshranBuddy.db.profile.EventTimer.relativePoint = relativePoint
	AshranBuddy.db.profile.EventTimer.x = x
	AshranBuddy.db.profile.EventTimer.y = y
end

function EventTimer:Unlock()
	self.containerFrame:Show()
	self.handle:Show()
end

function EventTimer:InitializeComponent(width, height)
	self.statusBars = { }
	IsDirty = false
	
	local profile = AshranBuddy.db.profile.EventTimer
	
	local frame = self.containerFrame
	frame:SetParent(UIParent)
	frame:SetSize(width, height)
	frame:SetPoint(profile.point, UIParent, profile.relativePoint, profile.x, profile.y)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetBackdrop({ bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background" })
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetToplevel(true)
	
	local handle = CreateFrame("Frame", nil, frame)
	handle:SetSize(width, height)
	handle:SetPoint("TOPLEFT", frame)
	handle:Hide()

	local handleBox = handle:CreateTexture(nil, "OVERLAY", nil, -1)
	handleBox:SetTexture(0, 0, 0, 0.6)
	handleBox:SetSize(width, height)
	handleBox:SetPoint("TOPLEFT")
	
	local handleFontString = handle:CreateFontString("HandleLabel", "OVERLAY", "TextStatusBarText")
	handleFontString:SetPoint("CENTER")
	handleFontString:SetText(L["Ashran Buddy\n\nYou MUST Type /ab lock or\n/ashranbuddy lock to activate.\nBars won't show until event is announced"])

	-- Children
	self.handle = handle
	local offset = -5
	for key, value in pairs(Event) do
		self.statusBars[#self.statusBars + 1] = self:CreateStatusBarFrame(frame, value.ID, value.Title, value.MapX, value.MapY, offset)
		offset = offset - 15
	end
	
	-- Events
	frame:SetScript("OnUpdate", EventTimer.OnUpdate)
	frame:SetScript("OnMouseDown", EventTimer.OnMouseDown)
	frame:SetScript("OnMouseUp", EventTimer.OnMouseUp)
end

function EventTimer:CreateStatusBarFrame(parent, id, title, mapX, mapY, offset)
	local statusBar = CreateFrame("StatusBar", nil, parent)
	statusBar:SetWidth(200)
	statusBar:SetHeight(10)
	statusBar:SetPoint("TOPLEFT", parent, "TOPLEFT", 5, offset)
	statusBar:SetMinMaxValues(0, 180)
	statusBar:SetValue(0)
	statusBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
	statusBar:SetStatusBarColor(0.26, 0.42, 1)

	-- Artwork Layer
	local borderLeft = statusBar:CreateTexture("BorderLeft", "ARTWORK")
	borderLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderLeft:SetTexCoord(0.007843, 0.043137, 0.193548, 0.774193)
	borderLeft:SetPoint("LEFT", -3, 0)
	borderLeft:SetSize(9, 14)

	local borderRight = statusBar:CreateTexture("BorderRight", "ARTWORK")
	borderRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderRight:SetTexCoord(0.043137, 0.007843, 0.193548, 0.774193)
	borderRight:SetPoint("RIGHT", 3, 0)
	borderRight:SetSize(9, 14)
	
	local borderMid = statusBar:CreateTexture("BorderMid", "ARTWORK")
	borderMid:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderMid:SetTexCoord(0.113726, 0.1490196, 0.193548, 0.774193)
	borderMid:SetPoint("TOPLEFT", borderLeft, "TOPRIGHT")
	borderMid:SetPoint("BOTTOMRIGHT", borderRight, "BOTTOMLEFT")
	borderMid:SetSize(9, 14)

	statusBar.eventNameFontString = statusBar:CreateFontString("EventNameLabel", "ARTWORK", "TextStatusBarText")
	statusBar.eventNameFontString:SetPoint("LEFT", statusBar, "LEFT", 8, 0)

	statusBar.eventTimeFontString = statusBar:CreateFontString("EventTimeLabel", "ARTWORK", "TextStatusBarText")
	statusBar.eventTimeFontString:SetPoint("RIGHT")
	
	if (not AshranBuddy.db.profile.EventTimer.showTimer) then
		eventTimeFontString:Hide()
	end
	
	-- Background Layer
	local barFull = statusBar:CreateTexture("BarFull", "BACKGROUND")
	barFull:SetVertexColor(0.04, 0.07, 0.18)
	
	statusBar.eventID = id
	statusBar.state = EventState.Hidden
	statusBar.mapX = mapX
	statusBar.mapY = mapY
	
	statusBar.GetEventID = function(self)
		return self.eventID
	end
	statusBar.GetEventTitle = function(self)
		return self.eventNameFontString:GetText()
	end
	statusBar.SetEventTitle = function(self, value)
		self.eventNameFontString:SetText(value)
	end
	statusBar.SetEventTime = function(self, value)
		self.eventTimeFontString:SetText(value)
	end
	statusBar.GetOrder = function(self)
		return ((self.state.Category * 100000) - self:GetValue())
	end
	statusBar.GetState = function(self)
		return self.state
	end
	statusBar.SetState = function(self, state)
		self.state = state
		if (self.eventTimer ~= nil) then
			EventTimer:CancelTimer(self.eventTimer)
			self.eventTimer = nil
		end
		if (state == EventState.Hidden) then
			self:SetEventTime("")
			self:SetMinMaxValues(state.MinValue, state.MaxValue)
			self:SetValue(state.Value)
			self:Hide()
		end
		if (AshranBuddy:IsInAshran()) then	
			local timeLeft = state.MaxValue - state.Value
			local text = state.Text
			if (text == nil) then
				text = string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60)
			end
			
			self:SetEventTime(text)
			self:SetMinMaxValues(state.MinValue, state.MaxValue)
			self:SetValue(state.Value)
			self:SetStatusBarColor(state.Red, state.Green, state.Blue)
		
			if (EventTransition[state] ~= nil) then
				self.eventTimer = EventTimer:ScheduleRepeatingTimer("OnTimerCallback", 1, self, EventTransition[state])
			end
			
			self:Show()
		end
		IsDirty = true
	end

	statusBar:SetScript("OnMouseDown", EventTimer.OnStatusBarClick)
	
	statusBar:SetEventTitle(title)
	statusBar:SetEventTime("")
	statusBar:Hide()
	
	return statusBar
end
function EventTimer:FindWorldMapLandmark(eventName, mapX, mapY) 
	local count = GetNumMapLandmarks()
	for i = 1, count do
        local worldMapPOIName = "WorldMapFramePOI" .. i
        local worldMapPOI = _G[worldMapPOIName]
		
		local name, description, textureIndex, x, y, mapLinkID, inBattleMap, graveyardID, areaID, poiID, isObjectIcon = GetMapLandmarkInfo(i)
		if (textureIndex == LandmarkTextureIndex and mapX == round(x, 2) and mapY == round(y, 2)) then
			return true
		end
	end
	return false
end

function EventTimer:OnChatCommand(input)
	if (input == "unlock") then
		EventTimer:Unlock()
		AshranBuddy.db.profile.EventTimer.locked = false
	elseif (input == "lock") then
		EventTimer:Lock()
		AshranBuddy.db.profile.EventTimer.locked = true
	elseif (input == "sync") then
		self:SendRequest()
	end
end

function EventTimer:OnCreateOptions(options)
	options.EventTimer = {
		name = L["Event Timer"],
		type = "group",
		args = {
			header = {
				name = L["Event Timer"],
				type = "header",
				order = 0
			},
			headerDescription = {
				name = L["Tracks and synchronizes timers for all Ashran events."],
				type = "description",
				order = 1
			},
			enabled = {
				name = L["Enabled"],
				desc = L["Enables the Ashran Event Timer module."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.enabled = val; if (val) then self:Enable() else self:Disable() end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.enabled end,
				order = 2
			},
			enabledDescription = {
				name = L["Enables the Ashran Event Timer module."],
				type = "description",
				order = 3
			},
			locked = {
				name = L["Lock Event Timer"],
				desc = L["Locks and unlocks the event timer frame."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.locked = val; if (AshranBuddy.db.profile.EventTimer.locked) then self:Lock() else self:Unlock() end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.locked end,
				order = 4
			},
			lockedDescription = {
				name = L["Locks and unlocks the event timer frame."],
				type = "description",
				order = 5
			},
			showTimer = {
				name = L["Show Numbered Timer"],
				desc = L["Displays numbered timer on the right of the countdown bar."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.showTimer = val; if (AshranBuddy.db.profile.EventTimer.showTimer) then for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Show() end else for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Hide() end end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.showTimer end,
				order = 6
			},
			showTimerDescription = {
				name = L["Displays numbered timer on the right of the countdown bar."],
				type = "description",
				order = 7
			},
			stacked = {
				name = L["Stack Timers"],
				desc = L["Neatly stack and sort the timers."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.stacked = val; if (AshranBuddy.db.profile.EventTimer.showTimer) then for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Show() end else for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Hide() end end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.stacked end,
				order = 8
			},
			stackedDescription = {
				name = L["Neatly stack and sort the timers."],
				type = "description",
				order = 9
			},
		},
	}
end

function EventTimer:OnMessageReceived(packet, distribution, sender)
	-- A raid member has requested a synchronise
	if (packet.type == "request") then
		EventTimer:CommRequest(sender)
	end
	
	-- You requested a synchronise and here it is
	if (packet.type == "synchronise" and distribution == "WHISPER") then
		EventTimer:CommSynchronise(sender, packet.data)
	end
end

function EventTimer:SendRequest()
	if (IsInRaid()) then
		local packet = { type = "request", data = {} }
		self:SendMessage(packet, "RAID")
	end
end

function EventTimer:CommRequest(sender)
	if (EventTimer.enabled) then
		local packet = { type = "synchronise", data = { } }
		for i = 1, #EventTimer.statusBars do
			local statusBar = EventTimer.statusBars[i]
			packet.data[i] = { state = statusBar:GetState().ID, value = statusBar:GetValue() }
		end

		EventTimer:SendMessage(packet, "WHISPER", sender)
	end
end

function EventTimer:CommSynchronise(sender, data)
	if (data ~= nil) then
		local states = { }
		for key, value in pairs(EventState) do
			states[value.ID] = value
		end
		for i = 1, #EventTimer.statusBars do
			local state = states[data[i].state]
			if (state ~= EventState.Hidden) then
				local statusBar = EventTimer.statusBars[i]
				statusBar:SetState(state)
				statusBar:SetValue(data[i].value)
			end
		end
	end
end

function EventTimer:Migrate()
	if (AshranBuddy.db.profile.locked ~= nil) then
		local profile = AshranBuddy.db.profile
		
		profile.EventTimer.enabled = self.enabled
		profile.EventTimer.locked = AshranBuddy.db.profile.locked
		profile.EventTimer.showTimer = AshranBuddy.db.profile.showTimer
		profile.EventTimer.point = AshranBuddy.db.profile.point
		profile.EventTimer.relativePoint = AshranBuddy.db.profile.relativePoint
		profile.EventTimer.x = AshranBuddy.db.profile.x
		profile.EventTimer.y = AshranBuddy.db.profile.y
		
		profile.locked = nil
		profile.showTimer = nil
		profile.point = nil
		profile.relativePoint = nil
		profile.x = nil
		profile.y = nil
	end
end