﻿-- NemesisFinder.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["Nemesis Finder"]))
end

-- Registration
local NemesisFinder = AshranBuddy:RegisterModule("NemesisFinder")

local MaxRaidMembers = 40
local NemesisQuests = {
	[36964] = { ID = 36964, Title = "Nemesis: Orcslayer", Race = "Orc" },
	[36921] = { ID = 36921, Title = "Nemesis: Manslayer", Race = "Human" },
	[36960] = { ID = 36960, Title = "Nemesis: Death Stalker", Race = "Undead" },
	[36932] = { ID = 36932, Title = "Nemesis: Scourge of the Kaldorei", Race = "NightElf" },
	[36958] = { ID = 36958, Title = "Nemesis: Slayer of Sindorei", Race = "BloodElf" },
	[36962] = { ID = 36962, Title = "Nemesis: The Butcher", Race = "Tauren" },
	[36966] = { ID = 36966, Title = "Nemesis: Troll Hunter", Race = "Troll" },
	[36968] = { ID = 36968, Title = "Nemesis: Huojins Fall", Race = "Pandaren" },
	[36934] = { ID = 36934, Title = "Nemesis: Terror of the Tushui", Race = "Pandaren" },
	[36930] = { ID = 36930, Title = "Nemesis: Draenei Destroyer", Race = "Draenei" },
	[36928] = { ID = 36928, Title = "Nemesis: Worgen Hunter", Race = "Worgen" },
	[36926] = { ID = 36926, Title = "Nemesis: Gnomebane", Race = "Gnome" },
	[36970] = { ID = 36970, Title = "Nemesis: Killer of Kezan", Race = "Goblin" },
	[36924] = { ID = 36924, Title = "Nemesis: Dwarfstalker", Race = "Dwarf" },
}

NemesisFinder.quest = nil
NemesisFinder.buttons = { }
NemesisFinder.members = { }
NemesisFinder.nemesis = { }

local platePool = { }

function NemesisFinder:OnInitialize()
	self:Migrate()
	self.quest = self:GetNemesisQuest()
end

function NemesisFinder:OnEnable()
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "OnGroupRosterUpdate")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "OnUpdateMouseOverUnit")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnPlayerTargetChanged")
	
	self:OnGroupRosterUpdate()
	
	self.containerFrame:SetScript("OnUpdate", function(self, elapsed)
		NemesisFinder:HookFrames(WorldFrame:GetChildren())
	end)
end

function NemesisFinder:OnDisable()
	self:UnregisterAllEvents()
	for i = 1, GetNumGroupMembers() do
		self.buttons[i]:UnlockHighlight()
	end
	
	self.containerFrame:SetScript("OnUpdate", nil)
end

function NemesisFinder:Setup()
	if (self.quest == nil) then
		self.quest = self:GetNemesisQuest()
	end
	if (#self.buttons == 0) then
		for i = 1, MaxRaidMembers do
			self.buttons[i] = _G["RaidGroupButton"..i]
		end
	end
end

function NemesisFinder:OnGroupRosterUpdate()
	self:Setup()
	if (self.quest ~= nil) then
		-- Reset Nemesis highlight
		self:RemoveHighlight()
		self:SendRequest()
	end
end

function NemesisFinder:OnPlayerEnteringWorld()
	self:OnGroupRosterUpdate()
end

function NemesisFinder:AddHighlight(name)
	if (name ~= nil) then
		local found = false
		for i = 1, GetNumGroupMembers() do
			if (self.buttons[i].name ~= nil) then
				if (string.find(self.buttons[i].name, name)) then
					self.buttons[i]:LockHighlight()
					break
				end
			end
		end
	end
end

function NemesisFinder:RemoveHighlight()
	for i = 1, GetNumGroupMembers() do
		if (self.buttons[i] ~= nil and self.buttons[i].name ~= nil) then
			if (not self:FindMember(self.buttons[i].name)) then
				self.buttons[i]:UnlockHighlight()
			end
		end
	end
end

function NemesisFinder:FindMember(name)
	local found = false
	for i = 1, #NemesisFinder.members do
		if (string.find(NemesisFinder.members[i], name)) then
			found = true
			break
		end
	end
	return found
end

function NemesisFinder:OnUpdateMouseOverUnit()
	self:OnNemesisMarked("mouseover")
end

function NemesisFinder:OnPlayerTargetChanged()
	self:OnNemesisMarked("target")
end

function NemesisFinder:OnNemesisMarked(unitID)
	if (self.quest ~= nil and UnitIsEnemy("player", unitID)) then
		local _, race = UnitRace(unitID)
		if (self.quest.Race == race) then
			local name, _ = UnitName(unitID)
			self:AddTarget(name)
		end
	end
end

function NemesisFinder:AddTarget(name)
	if (name ~= nil) then
		self.nemesis[name] = 1
		self:HookFrames(WorldFrame:GetChildren())
	end
end

function NemesisFinder:FindTarget(name)
	for key, value in pairs(NemesisFinder.nemesis) do
		if (name:find(key)) then
			return true
		end
	end
	return false
end

function NemesisFinder:HookFrames(...)
	for i = 1, select("#", ...) do
		local frame = select(i, ...)
		if (frame:GetName() ~= nil and frame:GetName():find("NamePlate")) then
			local child = select(2, frame:GetChildren())
			if (child ~= nil) then
				local region = child:GetRegions()
				if (region ~= nil and region:GetObjectType() == "FontString") then
					if (self:FindTarget(region:GetText())) then
						if (frame.Nemesis == nil) then
							frame.Nemesis = self:ObtainPlateFrame(frame)
							frame.Nemesis:SetParent(frame)
							frame.Nemesis:SetPoint("BOTTOM", frame, "TOP")
							frame.Nemesis:Show()
						end
					else
						if (frame.Nemesis ~= nil) then
							local nemesis = frame.Nemesis
							frame.Nemesis:Hide()
							frame.Nemesis:SetParent(nil)
							frame.Nemesis:ClearAllPoints()
							frame.Nemesis = nil
							
							table.insert(platePool, nemesis)
						end
					end
				end
			end
		end
	end
end

function NemesisFinder:OnPlateShow(frame)
	-- Add frame
	if (frame.Nemesis == nil) then
		local child = select(2, frame:GetChildren())
		if (child ~= nil) then
			local region = child:GetRegions()
			if (region ~= nil and region:GetObjectType() == "FontString" and self:FindTarget(region:GetText())) then
				frame.Nemesis = self:ObtainPlateFrame(frame)
				frame.Nemesis:SetParent(frame)
				frame.Nemesis:SetPoint("BOTTOM", frame, "TOP")
				frame.Nemesis:Show()
			end
		end
	end
end

function NemesisFinder:OnPlateHide(frame)
	-- Recycle frame
	if (frame.Nemesis ~= nil) then
		local nemesis = frame.Nemesis
		frame.Nemesis:Hide()
		frame.Nemesis:SetParent(nil)
		frame.Nemesis:ClearAllPoints()
		frame.Nemesis = nil
		
		table.insert(platePool, nemesis)
	end
end

function NemesisFinder:ObtainPlateFrame(frame)
	if (#platePool > 0) then
		return table.remove(platePool)
	else
		local nemesis = CreateFrame("Frame", nil, frame)
		nemesis:SetPoint("BOTTOM", frame, "TOP")
		nemesis:SetWidth(32)
		nemesis:SetHeight(32)
		
		nemesis.Texture = nemesis:CreateTexture(nil, "BACKGROUND")
		nemesis.Texture:SetTexture("Interface\\AddOns\\AshranBuddy\\Content\\Nemesis")
		nemesis.Texture:SetWidth(32)
		nemesis.Texture:SetHeight(32)
		nemesis.Texture:SetPoint("TOPLEFT")
		return nemesis
	end
end

function NemesisFinder:OnChatCommand(input)
end

function NemesisFinder:GetNemesisQuest()
	local nemesisQuest = nil
	local count = GetNumQuestLogEntries()
	for i = 1, count do
        local _, _, _, _, _, _, _, ID, _, _, _, _, _, _ = GetQuestLogTitle(i)
		if (NemesisQuests[ID] ~= nil) then
			nemesisQuest = NemesisQuests[ID]
			break
		end
	end
	return nemesisQuest
end

function NemesisFinder:OnMessageReceived(packet, distribution, sender)
	-- A raid member has requested a synchronise
	if (packet.type == "request") then
		NemesisFinder:CommRequest(sender, packet.data.quest)
	end
	
	-- You requested a synchronise and here it is
	if (packet.type == "synchronise" and distribution == "WHISPER") then
		NemesisFinder:CommSynchronise(sender)
	end
end

function NemesisFinder:SendRequest()
	if (IsInRaid()) then
		local packet = { type = "request", data = { quest = self.quest.ID } }
		-- Send the nemesis quest that I am currently on
		self:SendMessage(packet, "RAID")
	end
end

function NemesisFinder:CommRequest(sender, quest)
	local packet = { type = "synchronise", data = { } }
	-- If I have the same nemesis quest then respond otherwise ignore
	if (self.quest.ID == quest) then
		NemesisFinder:SendMessage(packet, "WHISPER", sender)
	end
end

function NemesisFinder:CommSynchronise(sender)
	NemesisFinder:AddHighlight(sender)
	table.insert(NemesisFinder.members, sender)
end

function NemesisFinder:Migrate()
	if (AshranBuddy.db.profile.NemesisFinder == nil) then
		local profile = AshranBuddy.db.profile
		profile.NemesisFinder = { }
		profile.NemesisFinder.enabled = true
	end
end

function NemesisFinder:OnCreateOptions(options)
	options.NemesisFinder = {
		name = L["Nemesis Finder"],
		type = "group",
		args = {
			header = {
				name = L["Nemesis Finder"],
				type = "header",
				order = 0
			},
			headerDescription = {
				name = L["Highlights all players in your current raid group that share the same Nemesis Quest as you. (Requires other players to have AshranBuddy)"],
				type = "description",
				order = 1
			},
			enabled = {
				name = L["Enabled"],
				desc = L["Enables the Nemesis Finder module."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.NemesisFinder.enabled = val; if (val) then self:Enable() else self:Disable() end end,
				get = function(info) return AshranBuddy.db.profile.NemesisFinder.enabled end,
				order = 2
			},
			enabledDescription = {
				name = L["Enables the Nemesis Finder module."],
				type = "description",
				order = 3
			},
			marked = {
				name = L["Nemesis Marking"],
				desc = L["Enables the marking of your selected nemesis race."],
				type = "toggle",
				set = function(info, val)
					AshranBuddy.db.profile.NemesisFinder.marking = val
					if (val) then
						self.containerFrame:SetScript("OnUpdate", function(self, elapsed)
							NemesisFinder:HookFrames(WorldFrame:GetChildren())
						end)
					else
						self.containerFrame:SetScript("OnUpdate", nil)
					end
				end,
				get = function(info) return AshranBuddy.db.profile.NemesisFinder.marking end,
				order = 2
			},
			markedDescription = {
				name = L["Enables the marking of your selected nemesis race."],
				type = "description",
				order = 3
			},
		},
	}
end
